section[data-testid='stSidebar'] {
    background-color: #111;
    flex-shrink: unset !important;
    
}

@media(hover:hover) and (min-width: 600px) and (max-width: 769px){

    header[data-testid="stHeader"] {
        display:none;
    }

    section[data-testid='stSidebar'] {
        height: 100%;
        min-width:95px !important;
        width: 95px !important;
        margin-left: 305px;
        position: relative;
        z-index: 1;
        top: 0;
        left: 0;
        background-color: #111;
        overflow-x: hidden;
        transition: 0.5s ease;
        padding-top: 60px;
        white-space: nowrap;
    }

    section[data-testid='stSidebar']:hover{
        min-width: 330px !important;
        }

    button[kind="header"] {
        display: none;
    }

    div[data-testid="collapsedControl"]{
        display: none;
    }

}

@media(hover: hover) and (min-width: 769px){

    header[data-testid="stHeader"] {
        display:none;
    }

    section[data-testid='stSidebar'] {
        height: 100%;
        min-width:95px !important;
        width: 95px !important;
        transform:translateX(0px);
        position: relative;
        z-index: 1;
        top: 0;
        left: 0;
        background-color: #111;
        overflow-x: hidden;
        transition: 0.5s ease;
        padding-top: 60px;
        white-space: nowrap;
    }

    section[data-testid='stSidebar']:hover{
        min-width: 330px !important;
        }

    button[kind="header"] {
        display: none;
    }

    div[data-testid="collapsedControl"]{
        display: none;
    }

.e1f1d6gn2 {
    text-align: center;
}

#render {
    margin: 10em auto 0;
    display: block;
    position: fixed;
    margin-top: 13em;
    border-radius: 0.7em;
    text-align: center;
}
}